@@ -43,12 +43,12 @@ run_updates() {
 	# regenerate dot of plugins
 	$SRC_DIR/scripts/dev/draw-all-plugins 2> $BASE_DIR/$VERSION/draw-all-plugins.error > $BASE_DIR/$VERSION/draw-all-plugins
 	git add $SRC_DIR/doc/images/plugins.*
-	git commit -a -m "release: regenerate plugins overview picture"
+	git commit -a -m "release: regenerate plugins overview picture" || true
 
 	# update info status
-	cd $SRC_DIR
-	$SCRIPTS_DIR/dev/update-infos-status 2> $BASE_DIR/$VERSION/update-infos-status.error > $BASE_DIR/$VERSION/update-infos-status
-	git commit -a -m "release: update plugin info status"
+	# cd $SRC_DIR
+	# $SCRIPTS_DIR/dev/update-infos-status 2> $BASE_DIR/$VERSION/update-infos-status.error > $BASE_DIR/$VERSION/update-infos-status
+	# git commit -a -m "release: update plugin info status" || true
 
 	# run link checker
 	cd $BUILD_DIR
