@@ -38,10 +38,10 @@ class Person : public ThreadString
 {
 public:
 	Person (KeySet & ks, ThreadContext & context_)
-	: ThreadString (ks, context_, Key ("/%layer1%/person", KEY_CASCADING_NAME, KEY_META, "default", "no name", KEY_END))
+	: ThreadString (ks, context_, Key ("/%layer1%/person", KEY_META, "default", "no name", KEY_END))
 	{
 	}
-	using ThreadString::operator= ;
+	using ThreadString::operator=;
 
 private:
 	Person () = delete;
@@ -53,23 +53,21 @@ class Nested : public ThreadInteger
 {
 public:
 	Nested (KeySet & ks, ThreadContext & context_)
-	: ThreadInteger (ks, context_,
-			 Key ("/test/%layer1%/%thread%/%layer2%/nested", KEY_CASCADING_NAME, KEY_META, "default", s_value, KEY_END))
+	: ThreadInteger (ks, context_, Key ("/test/%layer1%/%thread%/%layer2%/nested", KEY_META, "default", s_value, KEY_END))
 	{
 	}
-	using ThreadInteger::operator= ;
+	using ThreadInteger::operator=;
 };
 
 class Environment : public ThreadBoolean
 {
 public:
 	Environment (KeySet & ks, ThreadContext & context_)
-	: ThreadBoolean (ks, context_, Key ("/test/%layer1%", KEY_CASCADING_NAME, KEY_META, "default", "1", KEY_END)),
-	  nested (ks, context_), person (ks, context_),
-	  profile (ks, context_, Key ("/%layer1%/profile", KEY_CASCADING_NAME, KEY_META, "default", "default", KEY_END)),
+	: ThreadBoolean (ks, context_, Key ("/test/%layer1%", KEY_META, "default", "1", KEY_END)), nested (ks, context_),
+	  person (ks, context_), profile (ks, context_, Key ("/%layer1%/profile", KEY_META, "default", "default", KEY_END)),
 	  bm (ks, context_,
-	      Key ("/%layer1%/%layer2%/%layer3%/%layer4%/%layer5%/%layer6%/%layer7%/%layer8%/%layer9%/", KEY_CASCADING_NAME, KEY_META,
-		   "default", s_value, KEY_END))
+	      Key ("/%layer1%/%layer2%/%layer3%/%layer4%/%layer5%/%layer6%/%layer7%/%layer8%/%layer9%/", KEY_META, "default", s_value,
+		   KEY_END))
 	{
 	}
 
@@ -323,18 +321,18 @@ __attribute__ ((noinline)) void benchmark_contextual_noif_sum (kdb::Environment
 __attribute__ ((noinline)) void benchmark_hashmap ()
 {
 	std::unordered_map<std::string, kdb::ThreadInteger::type> hashmap;
-	hashmap["user/abc/test/hello"] = 5;
-	hashmap["user/abc/test/h1"] = 6;
-	hashmap["user/abc/test/h2"] = 7;
-	hashmap["user/abc/test/test"] = 9;
-	hashmap["user/abc/nested/hello"] = 12;
+	hashmap["user:/abc/test/hello"] = 5;
+	hashmap["user:/abc/test/h1"] = 6;
+	hashmap["user:/abc/test/h2"] = 7;
+	hashmap["user:/abc/test/test"] = 9;
+	hashmap["user:/abc/nested/hello"] = 12;
 	kdb::ThreadInteger::type x = 0;
 
 	static Timer t ("hashmap");
 	t.start ();
 	for (long long i = 0; i < iterations; ++i)
 	{
-		x ^= hashmap["user/abc/nested/notav"];
+		x ^= hashmap["user:/abc/nested/notav"];
 	}
 	t.stop ();
 	std::cout << t;
@@ -344,18 +342,18 @@ __attribute__ ((noinline)) void benchmark_hashmap ()
 __attribute__ ((noinline)) void benchmark_hashmap_find ()
 {
 	std::unordered_map<std::string, kdb::ThreadInteger::type> hashmap;
-	hashmap["user/abc/test/hello"] = 5;
-	hashmap["user/abc/test/h1"] = 6;
-	hashmap["user/abc/test/h2"] = 7;
-	hashmap["user/abc/test/test"] = 9;
-	hashmap["user/abc/nested/hello"] = 12;
+	hashmap["user:/abc/test/hello"] = 5;
+	hashmap["user:/abc/test/h1"] = 6;
+	hashmap["user:/abc/test/h2"] = 7;
+	hashmap["user:/abc/test/test"] = 9;
+	hashmap["user:/abc/nested/hello"] = 12;
 	kdb::ThreadInteger::type x = 0;
 
 	static Timer t ("hashmap find");
 	t.start ();
 	for (long long i = 0; i < iterations; ++i)
 	{
-		auto it = hashmap.find ("user/abc/nested/hello");
+		auto it = hashmap.find ("user:/abc/nested/hello");
 		if (it != hashmap.end ())
 		{
 			x ^= it->second;
@@ -467,13 +465,13 @@ __attribute__ ((noinline)) void benchmark_kslookup ()
 {
 	static Timer t ("kslookup");
 	using namespace kdb; // needed for KS_END
-	kdb::KeySet ks (100, *kdb::Key ("user/hello/some", KEY_END), *kdb::Key ("user/hello/a/key", KEY_END),
-			*kdb::Key ("user/hello/b/key", KEY_END), *kdb::Key ("user/hello/c/key", KEY_END),
-			*kdb::Key ("user/hello/d/key", KEY_END), *kdb::Key ("user/other", KEY_END), KS_END);
+	kdb::KeySet ks (100, *kdb::Key ("user:/hello/some", KEY_END), *kdb::Key ("user:/hello/a/key", KEY_END),
+			*kdb::Key ("user:/hello/b/key", KEY_END), *kdb::Key ("user:/hello/c/key", KEY_END),
+			*kdb::Key ("user:/hello/d/key", KEY_END), *kdb::Key ("user:/other", KEY_END), KS_END);
 	t.start ();
 	for (long long i = 0; i < iterations; ++i)
 	{
-		ks.lookup ("user/notfound");
+		ks.lookup ("user:/notfound");
 	}
 	t.stop ();
 	std::cout << t;
@@ -1085,7 +1083,7 @@ __attribute__ ((noinline)) void benchmark_layer_withN (kdb::Environment &, long
 	kdb::Coordinator c;
 	kdb::ThreadContext tc (c);
 	kdb::KeySet ks;
-	kdb::ThreadInteger ti (ks, tc, kdb::Key ("/test/nolayer", KEY_CASCADING_NAME, KEY_META, "default", s_value, KEY_END));
+	kdb::ThreadInteger ti (ks, tc, kdb::Key ("/test/nolayer", KEY_META, "default", s_value, KEY_END));
 	ti = 5;
 	kdb::ThreadInteger::type x = ti;
 
@@ -1097,8 +1095,8 @@ __attribute__ ((noinline)) void benchmark_layer_withN (kdb::Environment &, long
 		os << "/test/%layer1%/";
 		os << i;
 		// std::cout << os.str().c_str() << std::endl;
-		vi.push_back (std::make_shared<kdb::ThreadInteger> (
-			ks, tc, kdb::Key (os.str ().c_str (), KEY_CASCADING_NAME, KEY_META, "default", s_value, KEY_END)));
+		vi.push_back (std::make_shared<kdb::ThreadInteger> (ks, tc,
+								    kdb::Key (os.str ().c_str (), KEY_META, "default", s_value, KEY_END)));
 	}
 
 	static Timer * ts[10]{ new Timer ("layer with0"), new Timer ("layer with1"), new Timer ("layer with2"), new Timer ("layer with3"),
@@ -1144,7 +1142,7 @@ __attribute__ ((noinline)) void benchmark_layer_switchN (kdb::Environment &, lon
 	kdb::Coordinator c;
 	kdb::ThreadContext tc (c);
 	kdb::KeySet ks;
-	kdb::ThreadInteger ti (ks, tc, kdb::Key ("/test/nolayer", KEY_CASCADING_NAME, KEY_META, "default", s_value, KEY_END));
+	kdb::ThreadInteger ti (ks, tc, kdb::Key ("/test/nolayer", KEY_META, "default", s_value, KEY_END));
 	ti = 5;
 	kdb::ThreadInteger::type x = ti;
 
@@ -1156,8 +1154,8 @@ __attribute__ ((noinline)) void benchmark_layer_switchN (kdb::Environment &, lon
 		os << "/test/%layer1%/";
 		os << i;
 		// std::cout << os.str().c_str() << std::endl;
-		vi.push_back (std::make_shared<kdb::ThreadInteger> (
-			ks, tc, kdb::Key (os.str ().c_str (), KEY_CASCADING_NAME, KEY_META, "default", s_value, KEY_END)));
+		vi.push_back (std::make_shared<kdb::ThreadInteger> (ks, tc,
+								    kdb::Key (os.str ().c_str (), KEY_META, "default", s_value, KEY_END)));
 	}
 
 	static Timer * ts[10]{ new Timer ("layer switch0"), new Timer ("layer switch1"), new Timer ("layer switch2"),
