@@ -7,6 +7,7 @@
  */
 
 
+#include "kdblogger.h"
 #ifdef HAVE_KDBCONFIG_H
 #include "kdbconfig.h"
 #endif
@@ -98,7 +99,7 @@
  * easier to keyDup() the key.
  *
  * You can call keyNew() in many different ways depending on the attribute tags you
- * pass as parameters. Tags are represented as #keyswitch_t values, and
+ * pass as parameters. Tags are represented as #elektraKeyFlags values, and
  * tell keyNew() which Key attribute comes next.
  * The Key attribute tags are the following:
  * - ::KEY_VALUE \n
@@ -126,29 +127,10 @@
  *   So this flag toggle from keySetString() to keySetBinary().
  *   If no value (nor size) is given, it will be a NULL key.
  *   @snippet keyNew.c With Binary
- * - ::KEY_CASCADING_NAME allow the name to start with /
- *   useful for ksLookup() and kdbGet() parent/lookup keys
- * - ::KEY_META_NAME allow the name to start with arbitrary namespaces
- *   useful to compare with metakeys
- *
  *
  *
  * @deprecated The flags below are deprecated and ::KEY_META should be
  * preferred. They remain some time, however, for compatibility:
- * - ::KEY_DIR \n
- *   Define that the key is a directory rather than an ordinary key.
- *   This means its executable bits in its mode are set.
- *   But even without this option the key can have subkeys.
- *   See keySetDir().
- * - ::KEY_OWNER \n
- *   Next parameter is the owner. See keySetOwner().
- * - ::KEY_UID, ::KEY_GID \n
- *   Next parameter is taken as the UID (uid_t) or GID (gid_t) that will
- *   be defined on the key.
- *   See keySetUID() and keySetGID().
- * - ::KEY_MODE \n
- *   Next parameter is taken as mode permissions (int) to the key.
- *   See keySetMode().
  *   @snippet keyNew.c With Mode
  * - ::KEY_COMMENT \n
  *   Next parameter is a comment. See keySetComment().
@@ -166,56 +148,16 @@
  */
 Key * keyNew (const char * name, ...)
 {
-	Key * k;
+	if (!name) return NULL;
 
-	if (!name)
-	{
-		k = elektraCalloc (sizeof (Key));
-	}
-	else
-	{
-		va_list va;
-		va_start (va, name);
-		k = keyVNew (name, va);
-		va_end (va);
-	}
+	va_list va;
+	va_start (va, name);
+	Key * k = keyVNew (name, va);
+	va_end (va);
 
 	return k;
 }
 
-
-/**
- * @internal
- */
-static int elektraSetMetaInt (Key * key, const char * meta, int value)
-{
-	char * str = 0;
-	if ((str = elektraFormat ("%d", value)) == 0)
-	{
-		return -1;
-	}
-
-	keySetMeta (key, meta, str);
-	elektraFree (str);
-	return 0;
-}
-
-// duplicate of keySetMode in meta/meta.c
-static int elektraSetMode (Key * key, mode_t mode)
-{
-	char str[MAX_LEN_INT];
-	if (!key) return -1;
-
-	if (snprintf (str, MAX_LEN_INT - 1, "%o", mode) < 0)
-	{
-		return -1;
-	}
-
-	keySetMeta (key, "mode", str);
-
-	return 0;
-}
-
 /**
  * @copydoc keyNew
  *
@@ -224,317 +166,312 @@ static int elektraSetMode (Key * key, mode_t mode)
  */
 Key * keyVNew (const char * name, va_list va)
 {
+	if (!name) return NULL;
+
 	Key * key = elektraCalloc (sizeof (Key));
 
-	keyInit (key);
-	if (name)
+	elektraKeyFlags action = 0;
+	size_t value_size = 0;
+	void * value = 0;
+	void (*func) (void) = 0;
+	int flags = 0;
+
+	// flags that can be set via KEY_FLAGS
+	int allFlags = KEY_BINARY | KEY_LOCK_META | KEY_LOCK_NAME | KEY_LOCK_VALUE;
+
+	while ((action = va_arg (va, elektraKeyFlags)))
 	{
-		keyswitch_t action = 0;
-		size_t value_size = 0;
-		void * value = 0;
-		void (*func) (void) = 0;
-		int flags = 0;
-		char * owner = 0;
-		int mode = 0;
-		int hasMode = 0;
-
-		while ((action = va_arg (va, keyswitch_t)))
+		switch (action)
 		{
-			switch (action)
-			{
-			/* flags with an argument */
-			case KEY_SIZE:
-				value_size = va_arg (va, size_t);
-				break;
-			case KEY_VALUE:
-				value = va_arg (va, void *);
-				if (value_size && keyIsBinary (key))
-					keySetBinary (key, value, value_size);
-				else if (keyIsBinary (key))
-					keySetBinary (key, value, elektraStrLen (value));
-				else
-					keySetString (key, value);
-				break;
-			case KEY_FUNC:
-				func = va_arg (va, void (*) (void));
-				keySetBinary (key, &func, sizeof (func));
-				break;
-			case KEY_META:
-				value = va_arg (va, char *);
-				/* First parameter is name */
-				keySetMeta (key, value, va_arg (va, char *));
-				break;
+		/* flags with an argument */
+		case KEY_SIZE:
+			value_size = va_arg (va, size_t);
+			break;
+		case KEY_VALUE:
+			value = va_arg (va, void *);
+			if (value_size && keyIsBinary (key))
+				keySetBinary (key, value, value_size);
+			else if (keyIsBinary (key))
+				keySetBinary (key, value, elektraStrLen (value));
+			else
+				keySetString (key, value);
+			break;
+		case KEY_FUNC:
+			func = va_arg (va, void (*) (void));
+			keySetBinary (key, &func, sizeof (func));
+			break;
+		case KEY_META:
+			value = va_arg (va, char *);
+			/* First parameter is name */
+			keySetMeta (key, value, va_arg (va, char *));
+			break;
 
-			/* flags without an argument */
-			case KEY_FLAGS:
-				flags |= va_arg (va, int); // FALLTHROUGH
-			case KEY_BINARY:
-			case KEY_LOCK_NAME:
-			case KEY_LOCK_VALUE:
-			case KEY_LOCK_META:
-			case KEY_CASCADING_NAME:
-			case KEY_META_NAME:
-			case KEY_EMPTY_NAME:
-				if (action != KEY_FLAGS) flags |= action;
-				if (test_bit (flags, KEY_BINARY)) keySetMeta (key, "binary", "");
-				break;
+		/* flags without an argument */
+		case KEY_FLAGS:
+			flags |= (va_arg (va, int) & allFlags);
+			if (test_bit (flags, KEY_BINARY)) keySetMeta (key, "binary", "");
+			break;
+		case KEY_BINARY:
+			keySetMeta (key, "binary", ""); // FALLTHROUGH
+		case KEY_LOCK_NAME:
+		case KEY_LOCK_VALUE:
+		case KEY_LOCK_META:
+			flags |= action;
+			break;
 
-			/* deprecated flags */
-			case KEY_NAME:
-				name = va_arg (va, char *);
-				break;
-			case KEY_OWNER:
-				owner = va_arg (va, char *);
-				break;
-			case KEY_COMMENT:
-				keySetMeta (key, "comment", va_arg (va, char *));
-				break;
-			case KEY_UID:
-				elektraSetMetaInt (key, "uid", va_arg (va, int));
-				break;
-			case KEY_GID:
-				elektraSetMetaInt (key, "gid", va_arg (va, int));
-				break;
-			case KEY_DIR:
-				mode |= KDB_DIR_MODE;
-				break;
-			case KEY_MODE:
-				hasMode = 1;
-				mode |= va_arg (va, int);
-				break;
-			case KEY_ATIME:
-				elektraSetMetaInt (key, "atime", va_arg (va, time_t));
-				break;
-			case KEY_MTIME:
-				elektraSetMetaInt (key, "mtime", va_arg (va, time_t));
-				break;
-			case KEY_CTIME:
-				elektraSetMetaInt (key, "ctime", va_arg (va, time_t));
-				break;
+		/* deprecated flags */
+		case KEY_NAME:
+			name = va_arg (va, char *);
+			break;
+		case KEY_COMMENT:
+			keySetMeta (key, "comment", va_arg (va, char *));
+			break;
 
-			default:
-				ELEKTRA_ASSERT (0, "Unknown option " ELEKTRA_UNSIGNED_LONG_LONG_F " in keyNew",
-						(kdb_unsigned_long_long_t) action);
-				break;
-			}
+		default:
+			ELEKTRA_ASSERT (0, "Unknown option " ELEKTRA_UNSIGNED_LONG_LONG_F " in keyNew", (kdb_unsigned_long_long_t) action);
+			break;
 		}
-
-		option_t name_options = flags & (KEY_CASCADING_NAME | KEY_META_NAME | KEY_EMPTY_NAME);
-		elektraKeySetName (key, name, name_options);
-
-		if (!hasMode && mode == KDB_DIR_MODE)
-			elektraSetMode (key, KDB_FILE_MODE | KDB_DIR_MODE);
-		else if (mode != 0)
-			elektraSetMode (key, mode);
-
-		if (owner) keySetOwner (key, owner);
-
-		(void) keyLock (key, flags);
 	}
+
+	if (keySetName (key, name) < 0)
+	{
+		ELEKTRA_LOG_WARNING ("Invalid name: %s", name);
+		elektraFree (key);
+		return NULL;
+	}
+
+	keyLock (key, flags);
 	return key;
 }
 
 /**
- * Return a duplicate of a key.
+ * Copy or clear a key.
  *
- * Memory will be allocated as needed for dynamic properties.
+ * Depending on the chosen @p flags keyCopy() only copies
+ * certain parts of @p source into @p dest.
  *
- * The new key will not be member of any KeySet and
- * will start with a new reference counter at 0. A
- * subsequent keyDel() will delete the key.
+ * * If #KEY_CP_NAME is set, the key name will be copied
+ *   from @p source to @p dest.
+ * * If #KEY_CP_META is set, the meta keys will be copied
+ *   from @p source to @p dest.
+ * * If #KEY_CP_VALUE is set, the key value will be copied
+ *   from @p source to @p dest.
+ *   Additionally, if @p source is a binary key (keyIsBinary()),
+ *   @p dest will also be
+ *   marked as binary. This means that even if #KEY_CP_META is
+ *   not set, the `binary` meta key will be copied with
+ *   #KEY_CP_VALUE.
+ * * If #KEY_CP_STRING is set, the key value will be copied
+ *   from @p source to @p dest, but only, if @p source is
+ *   _not_ a binary key (keyIsBinary()). If @p source is binary,
+ *   keyCopy() fails. If @p dest is binary, it will still be
+ *   marked as binary after the copy.
+ *   This cannot be used together with #KEY_CP_VALUE.
+ *   The main purpose of #KEY_CP_STRING is for copying _into_
+ *   known string keys. It ensure that you don't accidentally
+ *   convert string keys into binary keys.
  *
- * @code
-int f (const Key * source)
-{
-	Key * dup = keyDup (source);
-	// work with duplicate
-	keyDel (dup);
-	// everything related to dup is freed
-	// and source is unchanged
-}
- * @endcode
+ * There is also the shorthand #KEY_CP_ALL. It is equivalent
+ * to `KEY_CP_NAME | KEY_CP_VALUE | KEY_CP_META`,
+ * i.e. all key data supported by keyCopy() will be copied
+ * from @p source to @p dest.
  *
- * Like for a new key after keyNew() a subsequent ksAppendKey()
- * makes a KeySet take care of the lifecycle of the key.
- *
- * @code
-int g (const Key * source, KeySet * ks)
-{
-	Key * dup = keyDup (source);
-	// work with duplicate
-	ksAppendKey (ks, dup);
-	// ksDel(ks) will also free the duplicate
-	// source remains unchanged.
-}
- * @endcode
- *
- * Duplication of keys should be preferred to keyNew(),
- * because data like owner can be filled with a copy
- * of the key instead of asking the environment.
- * It can also be optimized in the checks, because the keyname
- * is known to be valid.
- *
- * @param source has to be an initialized source Key
- * @retval 0 failure or on NULL pointer
- * @return a fully copy of source on success
- * @see ksAppendKey(), keyDel(), keyNew()
- * @ingroup key
- */
-Key * keyDup (const Key * source)
-{
-	Key * dest = 0;
-
-	if (!source) return 0;
-
-	dest = keyNew (0, KEY_END);
-	if (!dest) return 0;
-
-	/* Copy the struct data */
-	*dest = *source;
-
-	/* get rid of properties bound to old key */
-	dest->ksReference = 0;
-	dest->flags = KEY_FLAG_SYNC;
-
-	/* prepare to set dynamic properties */
-	dest->key = dest->data.v = dest->meta = 0;
-
-	/* copy dynamic properties */
-	if (keyCopy (dest, source) == -1)
-	{
-		keyDel (dest);
-		return 0;
-	}
-
-	return dest;
-}
-
-
-/**
- * Copy or Clear a key.
- *
- * Most often you may prefer keyDup() which allocates
- * a new key and returns a duplication of another key.
- *
- * But when you need to copy into an existing key, e.g.
+ * Use this function when you need to copy into an existing key, e.g.
  * because it was passed by a pointer in a function
  * you can do so:
  *
  * @snippet keyCopy.c Basic Usage
  *
- * The reference counter will not be changed for
- * both keys. Affiliation to keysets
- * are also not affected.
+ * Most often you will want to duplicate an existing key.
+ * For this purpose the alias keyDup() exists. Calling
  *
- * The metadata will be duplicated for the destination
- * key. So it will not take much additional space, even
- * with lots of metadata.
+ * @snippet keyCopy.c Dup Key
  *
- * When you pass a NULL-pointer as source the
- * data of dest will be cleaned completely
- * (except reference counter, see keyClear()) and
- * you get a fresh dest key:
+ * is equivalent to
+ *
+ * @snippet keyCopy.c Duplicate Key
+ *
+ * The reference counter will not be changed for both keys.
+ * Affiliation to keysets are also not affected.
+ *
+ * Since metadata uses copy-on-write semantics there is only a
+ * constant memory cost to copying metadata.
+ *
+ * When you pass a NULL-pointer as @p source the pieces of @p dest
+ * specified by @p flags will be cleared.
+ *
+ * Calling `keyCopy (dest, NULL, KEY_CP_ALL)` is different from calling keyClear().
+ * The key will not be fully reset, the reference counter and internal flags
+ * will remain unchanged. Additionally, keyCopy() respects keyLock() state,
+ * while keyClear() always works.
  *
  * @snippet keyCopy.c Clear
  *
- * If you want to copy everything, except e.g. the value
- * you can use keyCopy() too:
+ * @pre dest must be a valid Key (created with keyNew)
+ * @pre source must be a valid Key or NULL
  *
- * @snippet keyCopy.c Copy Without Value
- *
- * Restrain from coping everything yourself, because it will lead to
- * wrong metadata and is not able to copy empty or cascading names:
- *
- * @snippet keyCopy.c Individual Copy
+ * @invariant Key name stays valid until delete
  *
+ * @post Value from Key source is written to Key dest
  *
  * @param dest the key which will be written to
  * @param source the key which should be copied
- *     or NULL to clean the destination key
+ *     or NULL to clear the data of @p dest
+ * @param flags specifies which parts of the key should be copied
+ * @see keyDup()
+ * @since 0.9.5
  * @ingroup key
- * @retval -1 on failure when a NULL pointer
- *     was passed for dest or a dynamic property could not
- *     be written. The content will be unmodified then.
- * @retval 0 when dest was cleaned
- * @retval 1 when source was successfully copied
- * @see keyDup() to get a duplication of a key
+ *
+ * @return @p dest
+ * @retval NULL on memory allocation problems
+ * @retval NULL when a part of @p dest that should be modified (e.g. name, value) was marked read-only,
+ *              e.g. the name of @p dest will be read-only if @p dest is part of a KeySet
+ * @retval NULL when @p dest is NULL
+ * @retval NULL when both #KEY_CP_VALUE and #KEY_CP_STRING are set in @p flags
+ * @retval NULL when both #KEY_CP_STRING is set in @p flags and @p source is a binary key (keyIsBinary())
  */
-int keyCopy (Key * dest, const Key * source)
+Key * keyCopy (Key * dest, const Key * source, elektraCopyFlags flags)
 {
-	if (!dest) return -1;
+	if (dest == NULL) return NULL;
 
-	if (test_bit (dest->flags, KEY_FLAG_RO_NAME) || test_bit (dest->flags, KEY_FLAG_RO_VALUE) ||
-	    test_bit (dest->flags, KEY_FLAG_RO_META))
+	if (test_bit (dest->flags, KEY_FLAG_RO_NAME) && test_bit (flags, KEY_CP_NAME)) return NULL;
+	if (test_bit (dest->flags, KEY_FLAG_RO_VALUE) && test_bit (flags, KEY_CP_VALUE)) return NULL;
+	if (test_bit (dest->flags, KEY_FLAG_RO_META) && test_bit (flags, KEY_CP_META)) return NULL;
+
+	if (test_bit (flags, KEY_CP_STRING) && test_bit (flags, KEY_CP_VALUE)) return NULL;
+
+	if (source == NULL)
 	{
-		return -1;
+		if (test_bit (flags, KEY_CP_NAME))
+		{
+			keySetName (dest, "/");
+		}
+		if (test_bit (flags, KEY_CP_VALUE))
+		{
+			keySetRaw (dest, NULL, 0);
+		}
+		if (test_bit (flags, KEY_CP_META))
+		{
+			ksClear (dest->meta);
+		}
+		return dest;
 	}
 
-	if (!source)
-	{
-		keyClear (dest);
-		return 0;
-	}
+	if (test_bit (flags, KEY_CP_STRING) && keyIsBinary (source)) return NULL;
 
-	// remember dynamic memory to be removed
-	char * destKey = dest->key;
-	void * destData = dest->data.c;
-	KeySet * destMeta = dest->meta;
+	if (source == dest) return dest;
+
+	// remember original data of dest
+	Key orig = *dest;
 
 	// duplicate dynamic properties
-	if (source->key)
+	if (test_bit (flags, KEY_CP_NAME))
 	{
-		dest->key = elektraStrNDup (source->key, source->keySize + source->keyUSize);
-		if (!dest->key) goto memerror;
-	}
-	else
-	{
-		dest->key = 0;
+		if (source->key != NULL)
+		{
+			dest->key = elektraStrNDup (source->key, source->keySize);
+			if (!dest->key) goto memerror;
+			dest->keySize = source->keySize;
+
+			ELEKTRA_ASSERT (source->ukey != NULL, "key != NULL but ukey == NULL");
+			dest->ukey = elektraStrNDup (source->ukey, source->keyUSize);
+			if (!dest->ukey) goto memerror;
+			dest->keyUSize = source->keyUSize;
+		}
+		else
+		{
+			dest->key = elektraStrDup ("/");
+			dest->keySize = 2;
+
+			dest->ukey = elektraMalloc (3);
+			dest->ukey[0] = KEY_NS_CASCADING;
+			dest->ukey[1] = '\0';
+			dest->ukey[2] = '\0';
+			dest->keyUSize = 3;
+		}
+		clear_bit (dest->flags, KEY_FLAG_MMAP_KEY);
 	}
 
-	if (source->data.v)
+	if (test_bit (flags, KEY_CP_STRING))
 	{
-		dest->data.v = elektraStrNDup (source->data.v, source->dataSize);
-		if (!dest->data.v) goto memerror;
-	}
-	else
-	{
-		dest->data.v = 0;
+		if (source->data.v != NULL)
+		{
+			dest->data.v = elektraStrNDup (source->data.v, source->dataSize);
+			if (!dest->data.v) goto memerror;
+			dest->dataSize = source->dataSize;
+
+			if (!test_bit (flags, KEY_CP_META) && keyIsBinary (source))
+			{
+				keySetMeta (dest, "binary", "");
+			}
+		}
+		else
+		{
+			dest->data.v = NULL;
+			dest->dataSize = 0;
+		}
+		clear_bit (dest->flags, KEY_FLAG_MMAP_DATA);
 	}
 
-	if (source->meta)
+	if (test_bit (flags, KEY_CP_VALUE))
 	{
-		dest->meta = ksDup (source->meta);
-		if (!dest->meta) goto memerror;
+		if (source->data.v != NULL)
+		{
+			dest->data.v = elektraStrNDup (source->data.v, source->dataSize);
+			if (!dest->data.v) goto memerror;
+			dest->dataSize = source->dataSize;
+
+			if (!test_bit (flags, KEY_CP_META) && keyIsBinary (source))
+			{
+				keySetMeta (dest, "binary", "");
+			}
+		}
+		else
+		{
+			dest->data.v = NULL;
+			dest->dataSize = 0;
+		}
+		clear_bit (dest->flags, KEY_FLAG_MMAP_DATA);
 	}
-	else
+
+	if (test_bit (flags, KEY_CP_META))
 	{
-		dest->meta = 0;
+		if (source->meta != NULL)
+		{
+			dest->meta = ksDup (source->meta);
+			if (!dest->meta) goto memerror;
+		}
+		else
+		{
+			dest->meta = 0;
+		}
 	}
 
 	// successful, now do the irreversible stuff: we obviously modified dest
 	set_bit (dest->flags, KEY_FLAG_SYNC);
 
-	// copy sizes accordingly
-	dest->keySize = source->keySize;
-	dest->keyUSize = source->keyUSize;
-	dest->dataSize = source->dataSize;
-
 	// free old resources of destination
-	if (!test_bit (dest->flags, KEY_FLAG_MMAP_KEY)) elektraFree (destKey);
-	if (!test_bit (dest->flags, KEY_FLAG_MMAP_DATA)) elektraFree (destData);
-	ksDel (destMeta);
+	if (test_bit (flags, KEY_CP_NAME) && !test_bit (orig.flags, KEY_FLAG_MMAP_KEY)) elektraFree (orig.key);
+	if (test_bit (flags, KEY_CP_NAME) && !test_bit (orig.flags, KEY_FLAG_MMAP_KEY)) elektraFree (orig.ukey);
+	if (test_bit (flags, KEY_CP_VALUE) && !test_bit (orig.flags, KEY_FLAG_MMAP_DATA)) elektraFree (orig.data.c);
+	if (test_bit (flags, KEY_CP_META)) ksDel (orig.meta);
 
-	return 1;
+	return dest;
 
 memerror:
 	elektraFree (dest->key);
 	elektraFree (dest->data.v);
 	ksDel (dest->meta);
 
-	dest->key = destKey;
-	dest->data.v = destData;
-	dest->meta = destMeta;
-	return -1;
+	*dest = orig;
+	return NULL;
+}
+
+static void keyClearNameValue (Key * key)
+{
+	if (key->key && !test_bit (key->flags, KEY_FLAG_MMAP_KEY)) elektraFree (key->key);
+	if (key->ukey && !test_bit (key->flags, KEY_FLAG_MMAP_KEY)) elektraFree (key->ukey);
+	if (key->data.v && !test_bit (key->flags, KEY_FLAG_MMAP_DATA)) elektraFree (key->data.v);
 }
 
 
@@ -567,8 +504,6 @@ memerror:
  */
 int keyDel (Key * key)
 {
-	int rc;
-
 	if (!key) return -1;
 
 	if (key->ksReference > 0)
@@ -578,7 +513,7 @@ int keyDel (Key * key)
 
 	int keyInMmap = test_bit (key->flags, KEY_FLAG_MMAP_STRUCT);
 
-	rc = keyClear (key);
+	keyClearNameValue (key);
 
 	ksDel (key->meta);
 
@@ -587,7 +522,7 @@ int keyDel (Key * key)
 		elektraFree (key);
 	}
 
-	return rc;
+	return 0;
 }
 
 /**
@@ -632,15 +567,15 @@ int keyClear (Key * key)
 
 	int keyStructInMmap = test_bit (key->flags, KEY_FLAG_MMAP_STRUCT);
 
-	if (key->key && !test_bit (key->flags, KEY_FLAG_MMAP_KEY)) elektraFree (key->key);
-	if (key->data.v && !test_bit (key->flags, KEY_FLAG_MMAP_DATA)) elektraFree (key->data.v);
+	keyClearNameValue (key);
 
 	ksDel (key->meta);
 
 	keyInit (key);
-
 	if (keyStructInMmap) key->flags |= KEY_FLAG_MMAP_STRUCT;
 
+	keySetName (key, "/");
+
 	/* Set reference properties */
 	key->ksReference = ref;
 
@@ -787,7 +722,7 @@ ssize_t keyGetRef (const Key * key)
  * @retval -1 if it could not be locked (nullpointer)
  * @ingroup key
  */
-int keyLock (Key * key, option_t what)
+int keyLock (Key * key, elektraLockFlags what)
 {
 	if (!key) return -1;
 	what &= (KEY_LOCK_NAME | KEY_LOCK_VALUE | KEY_LOCK_META);
@@ -806,7 +741,7 @@ int keyLock (Key * key, option_t what)
  * @retval -1 on error (nullpointer)
  * @ingroup key
  */
-int keyIsLocked (const Key * key, option_t what)
+int keyIsLocked (const Key * key, elektraLockFlags what)
 {
 	if (!key) return -1;
 	what &= (KEY_LOCK_NAME | KEY_LOCK_VALUE | KEY_LOCK_META);
