@@ -348,7 +348,7 @@ KeySet * ksDeepDup (const KeySet * source)
 	for (i = 0; i < s; ++i)
 	{
 		Key * k = source->array[i];
-		Key * d = keyDup (k);
+		Key * d = keyDup (k, KEY_CP_ALL);
 		if (!test_bit (k->flags, KEY_FLAG_SYNC))
 		{
 			keyClearSync (d);
@@ -474,10 +474,11 @@ int ksDel (KeySet * ks)
  * @param ks the keyset object to work with
  * @see ksAppendKey() for details on how keys are inserted in KeySets
  * @retval 0 on success
- * @retval -1 on failure (memory)
+ * @retval -1 on failure (memory) or ks == NULL
  */
 int ksClear (KeySet * ks)
 {
+	if (ks == NULL) return -1;
 	ksClose (ks);
 	// ks->array empty now
 
@@ -936,6 +937,124 @@ ssize_t ksAppend (KeySet * ks, const KeySet * toAppend)
 	return ks->size;
 }
 
+/**
+ * The core rename loop of ksRename()
+ */
+static size_t ksRenameInternal (KeySet * ks, size_t start, size_t end, const Key * root, const Key * newRoot)
+{
+	for (size_t it = start; it < end; ++it)
+	{
+		if (ks->array[it]->ksReference == 1)
+		{
+			// only referenced in this KeySet -> just override read-only flag
+			clear_bit (ks->array[it]->flags, KEY_FLAG_RO_NAME);
+		}
+		else
+		{
+			// key has other references -> dup in-place so we can safely rename it
+			Key * dup = keyDup (ks->array[it], KEY_CP_ALL);
+			keyDecRef (ks->array[it]);
+			dup->ksReference = 1;
+			ks->array[it] = dup;
+		}
+		keyReplacePrefix (ks->array[it], root, newRoot);
+		// lock key with new name
+		set_bit (ks->array[it]->flags, KEY_FLAG_RO_NAME);
+	}
+	return end - start;
+}
+
+/**
+ * Moves all keys below @p root to below @p newRoot
+ *
+ * Only keys below @p root will be modified. The rest of
+ * @p ks remains untouched.
+ *
+ * This functions is similar to the following snippet, but
+ * there are some differences.
+ *
+ * @code{.c}
+ * KeySet * toRename = ksCut (ks, root);
+ * for (elektraCursor cursor = 0; cursor < ksGetSize (toRename); cursor++)
+ * {
+ *     Key * cur = keyDup (ksAtCursor (ks, cursor));
+ *     keyReplacePrefix (cur, root, newRoot);
+ *     ksAppendKey (ks, cur);
+ * }
+ * ksDel (toRename);
+ * @endcode
+ *
+ * Firstly, the optimizations only work, if @p ks doesn't
+ * contain any keys below @p newRoot that aren't below @p root.
+ * If such keys exist, ksRename() will still work, but it will fall
+ * back to code similar to the for-loop above.
+ *
+ * The second difference is that ksRename() will modify the keys in
+ * @p ks directly, if they aren't referenced from anywhere else
+ * (if their reference count is 1 (see keyGetRef())).
+ * Normally, this shouldn't cause problems, but if you have a direct
+ * `Key *` pointer to a key in @p ks or hold a reference to some data
+ * within a key of @p ks, you may need to call keyIncRef() to ensure
+ * the key isn't modified.
+ *
+ * @param ks      the keyset to manipulate
+ * @param root    the old prefix that will be removed, must not be a cascading key
+ * @param newRoot the new prefix the will replace the old one, must not be a cascading key
+ *
+ * @retval -1 if any of @p ks, @p root, @p newRoot is NULL, or if @p root or @p newRoot are cascading keys
+ * @retval -2 if @p ks already contains keys below @p newRoot
+ * @retval  0 if @p ks contains no keys below @p root (and also not @p root itself)
+ * @returns   otherwise, the number of keys that have been renamed
+ */
+ssize_t ksRename (KeySet * ks, const Key * root, const Key * newRoot)
+{
+	if (ks == NULL || root == NULL || newRoot == NULL) return -1;
+	if (keyGetNamespace (root) == KEY_NS_CASCADING || keyGetNamespace (newRoot) == KEY_NS_CASCADING) return -1;
+
+	// search the root
+	elektraCursor end;
+	size_t start = ksFindHierarchy (ks, root, &end);
+
+	// we found nothing
+	if (start == ks->size) return 0;
+
+	if (keyCmp (root, newRoot) == 0)
+	{
+		// same root, just return count
+		return end - start;
+	}
+
+	size_t newStart = ksFindHierarchy (ks, newRoot, NULL);
+	if (newStart < ks->size && keyIsBelowOrSame (newRoot, ks->array[newStart]) == 1)
+	{
+		// has keys below newRoot
+		if (start == newStart)
+		{
+			// first key below newRoot is also first key below root
+			// -> we can just rename keys and everything will be fine
+			// we don't need to re-sort/invalidate the hashmap,
+			// because the renamed subset is already in the right place
+			return ksRenameInternal (ks, start, end, root, newRoot);
+		}
+
+		// possible name collisions after rename
+		// -> ksCut and ksAppend to be safe
+		KeySet * toRename = ksCut (ks, root);
+		size_t renamed = ksRenameInternal (toRename, 0, ksGetSize (toRename), root, newRoot);
+		ksAppend (ks, toRename);
+		ksDel (toRename);
+		return renamed;
+	}
+
+	// rename everything below root
+	size_t renamed = ksRenameInternal (ks, start, end, root, newRoot);
+
+	// fix order and invalidate hashmap after renaming
+	qsort (ks->array, ks->size, sizeof (struct _Key *), keyCompareByNameOwner);
+	elektraOpmphmInvalidate (ks);
+
+	return renamed;
+}
 
 /**
  * @internal
@@ -975,6 +1094,78 @@ ssize_t ksCopyInternal (KeySet * ks, size_t to, size_t from)
 	return ret;
 }
 
+/**
+ * Searches for the start and optionally end of the key hierachy rooted at @p root in @p ks.
+ *
+ * The main use-case for this function is this kind of loop:
+ *
+ * @code{.c}
+ * elektraCursor end;
+ * for (elektraCursor it = ksFindHierarachy (ks, root, &end); it < end; ++it)
+ * {
+ * 	Key * cur = ksAtCursor (ks, it);
+ * }
+ * @endcode
+ *
+ * @param ks   The keyset to search in
+ * @param root The root of the hierachy to find
+ * @param end  If this is not NULL, it will be set to position of the first
+ *             key after @p root that is not below @p root. This is useful
+ *             for loops like the one above.
+ *             If not keys below @p root exist in @p ks, @p end will always
+ *             be set to the size of @p ks. This way a loop like the one above
+ *             will still work correctly.
+ *
+ * @retval -1 if @p ks or @p root are NULL
+ * @return the position of either @p root itself or the first key below
+ *         @p root that is part of @p ks. If no keys below @p root exist
+ *         in @p ks, the size of @p ks is returned. The snippet above
+ *         shows why this is useful.
+ */
+elektraCursor ksFindHierarchy (const KeySet * ks, const Key * root, elektraCursor * end)
+{
+	if (ks == NULL || root == NULL) return -1;
+
+	ssize_t search = ksSearchInternal (ks, root);
+	size_t it = search < 0 ? -search - 1 : search;
+	if (it == ks->size || keyGetNamespace (root) != keyGetNamespace (ks->array[it]) || keyIsBelowOrSame (root, ks->array[it]) != 1)
+	{
+		if (end != NULL)
+		{
+			*end = ks->size;
+		}
+		return ks->size;
+	}
+
+	if (end != NULL)
+	{
+		if (root->keyUSize == 3)
+		{
+
+			// special handling for root keys
+			// we just increment the namespace byte and search
+			// for the next theoretically possible namespace
+			root->ukey[0]++;
+			ssize_t endSearch = ksSearchInternal (ks, root);
+			root->ukey[0]--;
+			*end = endSearch < 0 ? -endSearch - 1 : endSearch;
+		}
+		else
+		{
+			// Very much a HACK to avoid allocating a new key name
+			// Overwriting the null terminator works fine, because
+			// all accesses to root->ukey inside of ksSearchInternal()
+			// use root->keyUSize explicitly.
+			root->ukey[root->keyUSize - 1] = '\1';
+			ssize_t endSearch = ksSearchInternal (ks, root);
+			root->ukey[root->keyUSize - 1] = '\0';
+			*end = endSearch < 0 ? -endSearch - 1 : endSearch;
+		}
+	}
+
+	return it;
+}
+
 /**
  * Searches for the start and end indicies corresponding to the given cutpoint.
  *
@@ -1785,7 +1976,7 @@ static Key * elektraLookupByCascading (KeySet * ks, Key * key, elektraLookupFlag
 		}
 
 		// we found a spec key, so we know what to do
-		specKey = keyDup (specKey);
+		specKey = keyDup (specKey, KEY_CP_ALL);
 		keySetBinary (specKey, keyValue (key), keyGetValueSize (key));
 		elektraCopyCallbackMeta (specKey, key);
 		found = elektraLookupBySpec (ks, specKey, options);
@@ -2119,7 +2310,7 @@ static Key * elektraLookupSearch (KeySet * ks, Key * key, elektraLookupFlags opt
 
 static Key * elektraLookupCreateKey (KeySet * ks, Key * key, ELEKTRA_UNUSED elektraLookupFlags options)
 {
-	Key * ret = keyDup (key);
+	Key * ret = keyDup (key, KEY_CP_ALL);
 	ksAppendKey (ks, ret);
 	return ret;
 }
@@ -2228,7 +2419,7 @@ Key * ksLookup (KeySet * ks, Key * key, elektraLookupFlags options)
 	if (options & KDB_O_SPEC)
 	{
 		Key * lookupKey = key;
-		if (test_bit (key->flags, KEY_FLAG_RO_NAME)) lookupKey = keyDup (key);
+		if (test_bit (key->flags, KEY_FLAG_RO_NAME)) lookupKey = keyDup (key, KEY_CP_NAME);
 		ret = elektraLookupBySpec (ks, lookupKey, options & mask);
 		if (test_bit (key->flags, KEY_FLAG_RO_NAME))
 		{
@@ -2239,7 +2430,7 @@ Key * ksLookup (KeySet * ks, Key * key, elektraLookupFlags options)
 	else if (!(options & KDB_O_NOCASCADING) && strcmp (name, "") && name[0] == '/')
 	{
 		Key * lookupKey = key;
-		if (test_bit (key->flags, KEY_FLAG_RO_NAME)) lookupKey = keyDup (key);
+		if (test_bit (key->flags, KEY_FLAG_RO_NAME)) lookupKey = keyDup (key, KEY_CP_NAME);
 		ret = elektraLookupByCascading (ks, lookupKey, options & mask);
 		if (test_bit (key->flags, KEY_FLAG_RO_NAME))
 		{
@@ -2436,9 +2627,12 @@ int ksInit (KeySet * ks)
  *
  * @see ksDel(), ksNew(), keyInit()
  * @retval 0 on success
+ * @retval -1 on ks == NULL
  */
 int ksClose (KeySet * ks)
 {
+	if (ks == NULL) return -1;
+
 	Key * k;
 
 	ksRewind (ks);
