@@ -102,33 +102,18 @@ int keyNeedSync (const Key * key)
 
 int keyIsSpec (const Key * key)
 {
-	if (!key) return -1;
-
-	if (key->key)
-		return keyNameIsSpec (key->key);
-	else
-		return 0;
+	return keyGetNamespace (key) == KEY_NS_SPEC;
 }
 
 int keyIsProc (const Key * key)
 {
-	if (!key) return -1;
-
-	if (key->key)
-		return keyNameIsProc (key->key);
-	else
-		return 0;
+	return keyGetNamespace (key) == KEY_NS_PROC;
 }
 
 
 int keyIsDir (const Key * key)
 {
-	if (!key) return -1;
-
-	if (key->key)
-		return keyNameIsDir (key->key);
-	else
-		return 0;
+	return keyGetNamespace (key) == KEY_NS_DIR;
 }
 
 
@@ -139,19 +124,13 @@ int keyIsDir (const Key * key)
  *
  * @param key the key object to work with
  * @retval 1 if key name begins with @p system, 0 otherwise
- * @retval -1 on NULL pointer
  * @see keyIsUser(), keySetName(), keyName()
  * @ingroup keytest
  *
  */
 int keyIsSystem (const Key * key)
 {
-	if (!key) return -1;
-
-	if (key->key)
-		return keyNameIsSystem (key->key);
-	else
-		return 0;
+	return keyGetNamespace (key) == KEY_NS_SYSTEM;
 }
 
 
@@ -162,19 +141,13 @@ int keyIsSystem (const Key * key)
  *
  * @param key the key object to work with
  * @retval 1 if key name begins with @p user, 0 otherwise
- * @retval -1 on NULL pointer
  * @see keyIsSystem(), keySetName(), keyName()
  * @ingroup keytest
  *
  */
 int keyIsUser (const Key * key)
 {
-	if (!key) return -1;
-
-	if (key->key)
-		return keyNameIsUser (key->key);
-	else
-		return 0;
+	return keyGetNamespace (key) == KEY_NS_USER;
 }
 
 /**
@@ -182,16 +155,16 @@ int keyIsUser (const Key * key)
  *
  * Example:
  @verbatim
- key user/sw/app
- check user/sw/app/key
+ key user:/sw/app
+ check user:/sw/app/key
  @endverbatim
  *
  * returns true because check is below key
  *
  * Example:
  @verbatim
- key user/sw/app
- check user/sw/app/folder/key
+ key user:/sw/app
+ check user:/sw/app/folder/key
  @endverbatim
  *
  * returns also true because check is indirect below key
@@ -222,8 +195,9 @@ int keyIsBelow (const Key * key, const Key * check)
 
 	// same key, only if namespace and size are equal
 	// size alone could be equal with cascading keys
-	return keyIsBelowOrSame (key, check) &&
-	       (keyGetNamespace (key) != keyGetNamespace (check) || keyGetUnescapedNameSize (key) != keyGetUnescapedNameSize (check));
+	return keyIsBelowOrSame (key, check) && keyGetUnescapedNameSize (key) != keyGetUnescapedNameSize (check) &&
+	       (keyGetNamespace (key) == keyGetNamespace (check) || keyGetNamespace (check) == KEY_NS_CASCADING ||
+		keyGetNamespace (key) == KEY_NS_CASCADING);
 }
 
 
@@ -246,32 +220,35 @@ int keyIsBelowOrSame (const Key * key, const Key * check)
 	size_t sizeAbove = keyGetUnescapedNameSize (key);
 	size_t sizeBelow = keyGetUnescapedNameSize (check);
 
-	if (sizeAbove == 1 && above[0] == '\0' && below[0] != '\0' && sizeBelow == strlen (below) + 1)
+	if ((sizeAbove == 3 && above[0] == KEY_NS_CASCADING && sizeBelow == 3 && below[0] != KEY_NS_CASCADING) ||
+	    (sizeBelow == 3 && below[0] == KEY_NS_CASCADING && sizeAbove == 3 && above[0] != KEY_NS_CASCADING))
 	{
-		// cascading root compared against other root
+		// cascading root compared to other root
 		return 0;
 	}
 
-	if (sizeBelow == 1 && below[0] == '\0' && above[0] != '\0' && sizeAbove == strlen (above) + 1)
+
+	if (sizeAbove == 3)
 	{
-		// cascading root compared against other root
-		return 0;
+		// root key, ignore trailing slash
+		sizeAbove -= 1;
 	}
 
-	if (above[0] != '\0' && below[0] == '\0')
+	if (sizeBelow == 3)
 	{
-		// cascading
-		size_t len = strlen (above);
-		above += len;
-		sizeAbove -= len;
+		// root key, ignore trailing slash
+		sizeBelow -= 1;
 	}
 
-	if (below[0] != '\0' && above[0] == '\0')
+	if ((above[0] != KEY_NS_CASCADING && below[0] == KEY_NS_CASCADING) ||
+	    (below[0] != KEY_NS_CASCADING && above[0] == KEY_NS_CASCADING))
 	{
-		// cascading
-		size_t len = strlen (below);
-		below += len;
-		sizeBelow -= len;
+		// cascading, ignore namespaces
+		++above;
+		--sizeAbove;
+
+		++below;
+		--sizeBelow;
 	}
 
 	if (sizeAbove > sizeBelow)
@@ -284,18 +261,18 @@ int keyIsBelowOrSame (const Key * key, const Key * check)
 
 
 /**
- * Check if the key check is direct below the key key or not.
+ * Check whether the key `check` is directly below the key `key`.
  *
  @verbatim
 Example:
-key user/sw/app
-check user/sw/app/key
+key user:/sw/app
+check user:/sw/app/key
 
 returns true because check is below key
 
 Example:
-key user/sw/app
-check user/sw/app/folder/key
+key user:/sw/app
+check user:/sw/app/folder/key
 
 does not return true, because there is only an indirect relation
 @endverbatim
@@ -322,22 +299,28 @@ int keyIsDirectlyBelow (const Key * key, const Key * check)
 	size_t sizeAbove = keyGetUnescapedNameSize (key);
 	size_t sizeBelow = keyGetUnescapedNameSize (check);
 
-	if (above[0] != '\0' && below[0] == '\0')
+	if (sizeAbove == 3)
 	{
-		// cascading
-		size_t len = strlen (above);
-		above += len;
-		sizeAbove -= len;
+		// root key, ignore trailing slash
+		sizeAbove -= 1;
 	}
 
-	if (below[0] != '\0' && above[0] == '\0')
+	if (sizeBelow == 3)
 	{
-		// cascading
-		size_t len = strlen (below);
-		below += len;
-		sizeBelow -= len;
+		// root key, ignore trailing slash
+		sizeBelow -= 1;
 	}
 
+	if ((above[0] != KEY_NS_CASCADING && below[0] == KEY_NS_CASCADING) ||
+	    (below[0] != KEY_NS_CASCADING && above[0] == KEY_NS_CASCADING))
+	{
+		// cascading, ignore namespaces
+		++above;
+		--sizeAbove;
+
+		++below;
+		--sizeBelow;
+	}
 	if (sizeAbove >= sizeBelow)
 	{
 		return 0;
@@ -347,53 +330,6 @@ int keyIsDirectlyBelow (const Key * key, const Key * check)
 	return memcmp (above, below, sizeAbove) == 0 && sizeAbove + nextPartSize + 1 == sizeBelow;
 }
 
-
-/**
- * Check whether a key is inactive.
- *
- * In Elektra terminology a hierarchy of keys is inactive if
- * the rootkey's basename starts with '.'. So a key is
- * also inactive if it is below an inactive key.
- * For example, user/key/.hidden is inactive and so
- * is user/.hidden/below.
- *
- * Inactive keys should not have any meaning to applications,
- * they are only a convention reserved for users and
- * administrators. To automatically remove all inactive keys
- * for an application, consider to use the hidden plugin.
- *
- * @param key the key object to work with
- * @retval 1 if the key is inactive
- * @retval 0 if the key is active
- * @retval -1 on NULL pointer or when key has no name
- * @ingroup keytest
- *
- */
-int keyIsInactive (const Key * key)
-{
-	if (!key) return -1;
-
-	const char * p = keyName (key);
-	if (!p) return -1;
-	if (p[0] == '\0') return -1;
-
-	size_t size = 0;
-
-	while (*(p = keyNameGetOneLevel (p + size, &size)))
-	{
-		if (size > 0)
-		{
-			if (p[0] == '.')
-			{
-				return 1;
-			}
-		}
-	}
-
-	return 0;
-}
-
-
 /**
  * Check if a key is binary type.
  *
@@ -449,12 +385,11 @@ int keyIsString (const Key * key)
  * Compare 2 keys.
  *
  * The returned flags bit array has 1s (differ) or 0s (equal) for each key
- * meta info compared, that can be logically ORed using @c #keyswitch_t flags.
- * @link keyswitch_t::KEY_NAME KEY_NAME @endlink,
- * @link keyswitch_t::KEY_VALUE KEY_VALUE @endlink,
- * @link keyswitch_t::KEY_OWNER KEY_OWNER @endlink,
- * @link keyswitch_t::KEY_COMMENT KEY_COMMENT @endlink,
- * @link keyswitch_t::KEY_META KEY_META @endlink (will be set in addition to owner and comment),
+ * meta info compared, that can be logically ORed using @c #elektraKeyFlags flags.
+ * @link elektraKeyFlags::KEY_NAME KEY_NAME @endlink,
+ * @link elektraKeyFlags::KEY_VALUE KEY_VALUE @endlink,
+ * @link elektraKeyFlags::KEY_COMMENT KEY_COMMENT @endlink,
+ * @link elektraKeyFlags::KEY_META KEY_META @endlink (will be set in addition to owner and comment),
  *
  * @par A very simple example would be
  * @code
@@ -470,9 +405,6 @@ if (changes == 0) printf("key1 and key2 are identicall\n");
 if (changes & KEY_VALUE)
 printf("key1 and key2 have different values\n");
 
-if (changes & KEY_UID)
-printf("key1 and key2 have different UID\n");
-
  *
  * @endcode
  *
@@ -489,23 +421,9 @@ printf("key1 and key2 have different UID\n");
 
  kdbGet(handle, ks, base);
 
-// we are interested only in key type and access permissions
-interests=(KEY_TYPE | KEY_MODE);
-
-ksRewind(ks);	// put cursor in the beginning
-while ((curren=ksNext(ks))) {
-match=keyCompare(current,base);
-
-if ((~match & interests) == interests)
-printf("Key %s has same type and permissions of base key",keyName(current));
-
-// continue walking in the KeySet....
-}
-
-// now we want same name and/or value
+// we are interested only in name and/or value
 interests=(KEY_NAME | KEY_VALUE);
 
-// we don't really need ksRewind(), since previous loop achieved end of KeySet
 ksRewind(ks);
 while ((current=ksNext(ks))) {
 match=keyCompare(current,base);
@@ -525,23 +443,21 @@ keyDel(base);
 * @return a bit array pointing the differences
 * @param key1 first key
 * @param key2 second key
-* @see #keyswitch_t
+* @see #elektraKeyFlags
 * @ingroup keytest
 	*/
-keyswitch_t keyCompare (const Key * key1, const Key * key2)
+elektraKeyFlags keyCompare (const Key * key1, const Key * key2)
 {
 	if (!key1 && !key2) return 0;
 	if (!key1 || !key2) return KEY_NULL;
 
-	keyswitch_t ret = 0;
+	elektraKeyFlags ret = 0;
 	ssize_t nsize1 = keyGetNameSize (key1);
 	ssize_t nsize2 = keyGetNameSize (key2);
 	const char * name1 = keyName (key1);
 	const char * name2 = keyName (key2);
 	const Key * comment1 = keyGetMeta (key1, "comment");
 	const Key * comment2 = keyGetMeta (key2, "comment");
-	const char * owner1 = keyOwner (key1);
-	const char * owner2 = keyOwner (key2);
 	const void * value1 = keyValue (key1);
 	const void * value2 = keyValue (key2);
 	ssize_t size1 = keyGetValueSize (key1);
@@ -550,8 +466,6 @@ keyswitch_t keyCompare (const Key * key1, const Key * key2)
 	// TODO: might be (binary) by chance
 	if (strcmp (keyString (comment1), keyString (comment2))) ret |= KEY_COMMENT;
 
-	if (strcmp (owner1, owner2)) ret |= KEY_OWNER;
-
 	if (keyCompareMeta (key1, key2)) ret |= KEY_META;
 
 	if (nsize1 != nsize2)
