@@ -113,7 +113,7 @@ keyDel(key);
  * value. Use keySetString() or keySetBinary() instead.
  *
  * @warning Binary keys will return a NULL pointer when there is no data in contrast
- * to keyName(), keyBaseName(), keyOwner() and keyComment(). For string value the
+ * to keyName(), keyBaseName() and keyComment(). For string value the
  * behaviour is the same.
  *
  * @par Example:
@@ -122,7 +122,7 @@ KDB *handle = kdbOpen();
 KeySet *ks=ksNew(0, KS_END);
 Key *current=0;
 
-kdbGetByName(handle,ks,"system/sw/my",KDB_O_SORT|KDB_O_RECURSIVE);
+kdbGetByName(handle,ks,"system:/sw/my",KDB_O_SORT|KDB_O_RECURSIVE);
 
 ksRewind(ks);
 while (current=ksNext(ks)) {
@@ -162,7 +162,6 @@ const void * keyValue (const Key * key)
 
 	if (!key->data.v)
 	{
-		/*errno=KDB_ERR_NOKEY;*/
 		if (keyIsBinary (key))
 			return 0;
 		else
@@ -246,7 +245,6 @@ ssize_t keyGetValueSize (const Key * key)
 
 	if (!key->data.v)
 	{
-		/*errno=KDB_ERR_NODATA;*/
 		if (keyIsBinary (key))
 			return 0;
 		else
@@ -269,7 +267,7 @@ ssize_t keyGetValueSize (const Key * key)
  *
  * @par Example:
  * @code
-Key *key = keyNew ("user/keyname", KEY_END);
+Key *key = keyNew ("user:/keyname", KEY_END);
 char buffer[300];
 
 if (keyGetString(key,buffer,sizeof(buffer)) == -1)
@@ -305,7 +303,6 @@ ssize_t keyGetString (const Key * key, char * returnedString, size_t maxSize)
 
 	if (!keyIsString (key))
 	{
-		/*errno=KDB_ERR_TYPEMISMATCH;*/
 		return -1;
 	}
 
@@ -317,7 +314,6 @@ ssize_t keyGetString (const Key * key, char * returnedString, size_t maxSize)
 
 	if (key->dataSize > maxSize)
 	{
-		/*errno=KDB_ERR_TRUNC;*/
 		return -1;
 	}
 
@@ -337,8 +333,6 @@ ssize_t keyGetString (const Key * key, char * returnedString, size_t maxSize)
  * called, in UTF-8 universal encoding, regardless of the program's current
  * encoding, when iconv plugin is present.
  *
- * @note The type will be set to KEY_TYPE_STRING.
- * When the type of the key is already a string type it won't be changed.
  *
  * @param key the key to set the string value
  * @param newStringValue NULL-terminated text string to be set as @p key's
@@ -386,7 +380,7 @@ ssize_t keySetString (Key * key, const char * newStringValue)
  *
  * @par Example:
  * @code
-Key *key = keyNew ("user/keyname", KEY_TYPE, KEY_TYPE_BINARY, KEY_END);
+Key *key = keyNew ("user:/keyname", KEY_BINARY, KEY_END);
 char buffer[300];
 
 if (keyGetBinary(key,buffer,sizeof(buffer)) == -1)
@@ -420,7 +414,6 @@ ssize_t keyGetBinary (const Key * key, void * returnedBinary, size_t maxSize)
 
 	if (!keyIsBinary (key))
 	{
-		/*errno=KDB_ERR_TYPEMISMATCH;*/
 		return -1;
 	}
 
@@ -431,7 +424,6 @@ ssize_t keyGetBinary (const Key * key, void * returnedBinary, size_t maxSize)
 
 	if (key->dataSize > maxSize)
 	{
-		/*errno=KDB_ERR_TRUNC;*/
 		return -1;
 	}
 
