@@ -69,7 +69,7 @@ static Key * splitEnvValue (const Key * envKey);
 
 static KeySet * ksMetaGetSingleOrArray (Key * key, const char * metaName);
 
-char * generateUsageLine (const char * progname, const Key * command, const Key * commandArgs);
+char * generateUsageLine (const char * progname, Key * command, const Key * commandArgs);
 static char * generateOptionsList (KeySet * keysWithOpts, Key * commandKey);
 static char * generateCommandsList (KeySet * keysWithOpts, Key * commandKey);
 static char * generateArgsList (KeySet * keysWithOpts, Key * commandKey);
@@ -111,9 +111,6 @@ static int writeOptions (Key * command, Key * commandKey, Key * commandArgs, boo
  * NOTE: Per default option processing DOES NOT stop, when a non-option string is encountered in @p argv.
  * If you want processing to stop, set the metadata `posixly = 1` on @p parentKey.
  *
- * The basic usage of this function is as follows:
- * @snippet optsSnippets.c basic use
- *
  *
  * @param ks	    The KeySet containing the specification for the options.
  * @param argc	    The number of strings in argv.
@@ -122,7 +119,7 @@ static int writeOptions (Key * command, Key * commandKey, Key * commandArgs, boo
  * 		    strings of the format 'KEY=VALUE'.
  * @param parentKey The parent key below which the function while search for option specifications.
  *                  Also used for error reporting. The key will be translated into the spec namespace
- *                  automatically, i.e. 'user/test/parent' will be translated into 'spec/test/parent',
+ *                  automatically, i.e. 'user:/test/parent' will be translated into 'spec:/test/parent',
  *                  before checking against spec keys.
  *
  * @retval 0	on success, this is the only case in which @p ks will be modified
@@ -132,19 +129,10 @@ static int writeOptions (Key * command, Key * commandKey, Key * commandArgs, boo
  */
 int elektraGetOpts (KeySet * ks, int argc, const char ** argv, const char ** envp, Key * parentKey)
 {
-	cursor_t initial = ksGetCursor (ks);
+	elektraCursor initial = ksGetCursor (ks);
 
-	Key * specParent = keyDup (parentKey);
-	if (keyGetNamespace (parentKey) != KEY_NS_SPEC)
-	{
-		keySetName (specParent, "spec");
-
-		const char * parent = strchr (keyName (parentKey), '/');
-		if (parent != NULL)
-		{
-			keyAddName (specParent, parent + 1);
-		}
-	}
+	Key * specParent = keyDup (parentKey, KEY_CP_ALL);
+	keySetNamespace (specParent, KEY_NS_SPEC);
 
 	struct Specification spec;
 	if (!processSpec (&spec, ks, specParent, parentKey))
@@ -188,7 +176,7 @@ int elektraGetOpts (KeySet * ks, int argc, const char ** argv, const char ** env
 				endArg += lastEndArg;
 
 				Key * commandSpec = ksLookup (spec.keys, commandKey, 0);
-				Key * commandLookup = keyNew ("command", KEY_META_NAME, KEY_END);
+				Key * commandLookup = keyNew ("meta:/command", KEY_END);
 				keyAddBaseName (commandLookup, argv[endArg]);
 				subCommand = ksLookup (keyMeta (commandSpec), commandLookup, KDB_O_DEL);
 			}
@@ -225,7 +213,7 @@ int elektraGetOpts (KeySet * ks, int argc, const char ** argv, const char ** env
 				return -1;
 			}
 
-			Key * procKey = keyNew ("proc", KEY_VALUE, "", KEY_END);
+			Key * procKey = keyNew ("proc:/", KEY_VALUE, "", KEY_END);
 			keyAddName (procKey, strchr (keyName (commandKey), '/'));
 			ksAppendKey (ks, procKey);
 
@@ -303,26 +291,26 @@ const char * elektraGetOptsHelpCommand (Key * errorKey)
 }
 
 /**
- * Extracts the help message from the @p errorKey used in elektraGetOpts().
+ * Extracts the help message from the @p helpKey used in elektraGetOpts().
  *
- * @param errorKey The same Key as passed to elektraGetOpts() as errorKey.
+ * @param helpKey  The same Key as passed to elektraGetOpts() as parentKey.
  * @param usage	   If this is not NULL, it will be used instead of the default usage line.
  * 		   Use elektraGetOptsHelpCommand() to check which command was invoked to get the right usage line.
  * @param prefix   If this is not NULL, it will be inserted between the usage line and the options list.
  *
- * @return The full help message extracted from @p errorKey, or NULL if no help message was found.
+ * @return The full help message extracted from @p helpKey, or NULL if no help message was found.
  * The returned string has to be freed with elektraFree().
  */
-char * elektraGetOptsHelpMessage (Key * errorKey, const char * usage, const char * prefix)
+char * elektraGetOptsHelpMessage (Key * helpKey, const char * usage, const char * prefix)
 {
-	const char * command = elektraGetOptsHelpCommand (errorKey);
+	const char * command = elektraGetOptsHelpCommand (helpKey);
 
 	Key * lookup;
 	if (usage == NULL)
 	{
-		lookup = keyNew ("internal/libopts/help/usage", KEY_META_NAME, KEY_END);
+		lookup = keyNew ("meta:/internal/libopts/help/usage", KEY_END);
 		keyAddBaseName (lookup, command);
-		usage = keyGetMetaStringByKey (errorKey, lookup);
+		usage = keyGetMetaStringByKey (helpKey, lookup);
 	}
 
 	if (usage == NULL)
@@ -330,33 +318,33 @@ char * elektraGetOptsHelpMessage (Key * errorKey, const char * usage, const char
 		return NULL;
 	}
 
-	lookup = keyNew ("internal/libopts/help/options", KEY_META_NAME, KEY_END);
+	lookup = keyNew ("meta:/internal/libopts/help/options", KEY_END);
 	keyAddBaseName (lookup, command);
-	const char * options = keyGetMetaStringByKey (errorKey, lookup);
+	const char * options = keyGetMetaStringByKey (helpKey, lookup);
 	if (options == NULL)
 	{
 		options = "";
 	}
 
-	lookup = keyNew ("internal/libopts/help/commands", KEY_META_NAME, KEY_END);
+	lookup = keyNew ("meta:/internal/libopts/help/commands", KEY_END);
 	keyAddBaseName (lookup, command);
-	const char * commands = keyGetMetaStringByKey (errorKey, lookup);
+	const char * commands = keyGetMetaStringByKey (helpKey, lookup);
 	if (commands == NULL)
 	{
 		commands = "";
 	}
 
-	lookup = keyNew ("internal/libopts/help/args", KEY_META_NAME, KEY_END);
+	lookup = keyNew ("meta:/internal/libopts/help/args", KEY_END);
 	keyAddBaseName (lookup, command);
-	const char * args = keyGetMetaStringByKey (errorKey, lookup);
+	const char * args = keyGetMetaStringByKey (helpKey, lookup);
 	if (args == NULL)
 	{
 		args = "";
 	}
 
-	lookup = keyNew ("internal/libopts/help/envs", KEY_META_NAME, KEY_END);
+	lookup = keyNew ("meta:/internal/libopts/help/envs", KEY_END);
 	keyAddBaseName (lookup, command);
-	const char * envs = keyGetMetaStringByKey (errorKey, lookup);
+	const char * envs = keyGetMetaStringByKey (helpKey, lookup);
 	if (envs == NULL)
 	{
 		envs = "";
@@ -406,7 +394,7 @@ bool processSpec (struct Specification * spec, KeySet * ks, Key * specParent, Ke
 	spec->argIndices = ksNew (0, KS_END);
 	spec->commands = ksNew (0, KS_END);
 
-	for (cursor_t i = 0; i < ksGetSize (ks); ++i)
+	for (elektraCursor i = 0; i < ksGetSize (ks); ++i)
 	{
 		Key * cur = ksAtCursor (ks, i);
 
@@ -520,7 +508,7 @@ bool processSpec (struct Specification * spec, KeySet * ks, Key * specParent, Ke
 			{
 				// add sub-command to parent command
 				Key * parentCommand = ksLookup (spec->keys, maybeCommand, 0);
-				Key * subCommand = keyNew ("command", KEY_META_NAME, KEY_VALUE, keyBaseName (cur), KEY_END);
+				Key * subCommand = keyNew ("meta:/command", KEY_VALUE, keyBaseName (cur), KEY_END);
 				keyAddBaseName (subCommand, keyString (commandMeta));
 
 				if (ksLookup (keyMeta (parentCommand), subCommand, 0) != NULL)
@@ -657,7 +645,7 @@ bool processOptions (struct Specification * spec, Key * command, Key * specKey,
 
 		// no other way to create Key with name "opt"
 		Key * k = keyNew ("/", KEY_META, "opt", "", KEY_END);
-		opts = ksNew (2, keyNew ("/#", KEY_END), keyGetMeta (k, "opt"), KS_END);
+		opts = ksNew (2, keyNew ("meta:/#", KEY_END), keyGetMeta (k, "opt"), KS_END);
 		keyDel (k);
 	}
 
@@ -751,7 +739,7 @@ bool processOptions (struct Specification * spec, Key * command, Key * specKey,
 bool readOptionData (struct OptionData * optionData, Key * key, const char * metaKey, Key * errorKey)
 {
 	// two slashes in string because array index is inserted in-between
-	char metaBuffer[ELEKTRA_MAX_ARRAY_SIZE + sizeof ("opt//flagvalue") + 1];
+	char metaBuffer[ELEKTRA_MAX_ARRAY_SIZE + sizeof ("meta:/opt//flagvalue") + 1];
 	strncpy (metaBuffer, metaKey, ELEKTRA_MAX_ARRAY_SIZE + 3); // 3 = opt/ - null byte from ELEKTRA_MAX_SIZE
 	strncat (metaBuffer, "/arg", 11);			   // 11 = remaining space in metaBuffer
 
@@ -1088,7 +1076,7 @@ bool processArgs (Key * command, Key * specKey, KeySet * argIndices, Key ** keyW
 		}
 		keySetMeta (*keyWithOpt, "args", "remaining");
 
-		Key * dup = keyDup (specKey);
+		Key * dup = keyDup (specKey, KEY_CP_ALL);
 		keySetBaseName (dup, NULL); // remove #
 
 		const char * existing = keyGetMetaString (command, "remainingargskey");
@@ -1322,21 +1310,21 @@ int writeArgsValues (KeySet * ks, Key * keyWithOpt, Key * command, KeySet * argI
 	{
 		Key * argIndex = ksLookup (argIndices, command, 0);
 		KeySet * indices = elektraMetaArrayToKS (argIndex, "index");
-		cursor_t firstRemainingArg = argIndex == NULL ? 0 : ksGetSize (indices) - 1; // -1 because of parent
+		elektraCursor firstRemainingArg = argIndex == NULL ? 0 : ksGetSize (indices) - 1; // -1 because of parent
 		ksDel (indices);
 
-		Key * procKey = keyNew ("proc", KEY_END);
+		Key * procKey = keyNew ("proc:/", KEY_END);
 		keyAddName (procKey, strchr (keyName (keyWithOpt), '/'));
 
-		Key * insertKey = keyDup (procKey);
+		Key * insertKey = keyDup (procKey, KEY_CP_NAME);
 
 		ksRewind (args);
-		for (cursor_t i = firstRemainingArg; i < ksGetSize (args); ++i)
+		for (elektraCursor i = firstRemainingArg; i < ksGetSize (args); ++i)
 		{
 			Key * arg = ksAtCursor (args, i);
 			elektraArrayIncName (insertKey);
 
-			Key * k = keyDup (insertKey);
+			Key * k = keyDup (insertKey, KEY_CP_NAME);
 			keySetString (k, keyString (arg));
 			ksAppendKey (ks, k);
 		}
@@ -1370,7 +1358,7 @@ int writeArgsValues (KeySet * ks, Key * keyWithOpt, Key * command, KeySet * argI
 			return -1;
 		}
 
-		Key * procKey = keyNew ("proc", KEY_END);
+		Key * procKey = keyNew ("proc:/", KEY_END);
 		keyAddName (procKey, strchr (keyName (keyWithOpt), '/'));
 		keySetString (procKey, keyString (arg));
 		ksAppendKey (ks, procKey);
@@ -1432,7 +1420,7 @@ int addProcKey (KeySet * ks, const Key * key, Key * valueKey)
 		return 1;
 	}
 
-	Key * procKey = keyNew ("proc", KEY_END);
+	Key * procKey = keyNew ("proc:/", KEY_END);
 	keyAddName (procKey, strchr (keyName (key), '/'));
 
 	bool isArrayKey = elektraStrCmp (keyBaseName (procKey), "#") == 0;
@@ -1455,7 +1443,7 @@ int addProcKey (KeySet * ks, const Key * key, Key * valueKey)
 
 	if (isArrayKey)
 	{
-		Key * insertKey = keyDup (procKey);
+		Key * insertKey = keyDup (procKey, KEY_CP_NAME);
 		keyAddBaseName (insertKey, "#");
 		KeySet * values = elektraMetaArrayToKS (valueKey, "values");
 		if (values == NULL)
@@ -1472,7 +1460,7 @@ int addProcKey (KeySet * ks, const Key * key, Key * valueKey)
 		{
 			elektraArrayIncName (insertKey);
 
-			Key * k = keyDup (insertKey);
+			Key * k = keyDup (insertKey, KEY_CP_NAME);
 			keySetString (k, keyString (cur));
 			ksAppendKey (ks, k);
 		}
@@ -1578,7 +1566,7 @@ KeySet * parseArgs (Key * command, KeySet * optionsSpec, bool useSubcommands, in
 			}
 
 			elektraArrayIncName (argKey);
-			Key * newArgKey = keyDup (argKey);
+			Key * newArgKey = keyDup (argKey, KEY_CP_NAME);
 			keySetString (newArgKey, cur);
 			ksAppendKey (options, newArgKey);
 		}
@@ -1588,7 +1576,7 @@ KeySet * parseArgs (Key * command, KeySet * optionsSpec, bool useSubcommands, in
 	for (; i < argc; ++i)
 	{
 		elektraArrayIncName (argKey);
-		Key * newArgKey = keyDup (argKey);
+		Key * newArgKey = keyDup (argKey, KEY_CP_NAME);
 		keySetString (newArgKey, argv[i]);
 		ksAppendKey (options, newArgKey);
 	}
@@ -1857,7 +1845,7 @@ int writeOptions (Key * command, Key * commandKey, Key * commandArgs, bool write
 	{
 		if (spec->useSubcommands)
 		{
-			Key * checkKey = keyDup (keyWithOpt);
+			Key * checkKey = keyDup (keyWithOpt, KEY_CP_ALL);
 			if (strcmp (keyBaseName (keyWithOpt), "#") == 0)
 			{
 				keySetBaseName (checkKey, NULL); // remove #
@@ -1874,7 +1862,7 @@ int writeOptions (Key * command, Key * commandKey, Key * commandArgs, bool write
 
 		if (keyGetMeta (keyWithOpt, "command") != NULL)
 		{
-			Key * procKey = keyNew ("proc", KEY_VALUE, "", KEY_END);
+			Key * procKey = keyNew ("proc:/", KEY_VALUE, "", KEY_END);
 			keyAddName (procKey, strchr (keyName (keyWithOpt), '/'));
 			ksAppendKey (ks, procKey);
 		}
@@ -1937,10 +1925,10 @@ KeySet * ksMetaGetSingleOrArray (Key * key, const char * metaName)
 	if (value[0] != '#')
 	{
 		// add dummy key to mimic elektraMetaArrayToKS
-		return ksNew (2, keyNew ("/#", KEY_END), k, KS_END);
+		return ksNew (2, keyNew ("meta:/#", KEY_END), k, KS_END);
 	}
 
-	Key * testKey = keyDup (k);
+	Key * testKey = keyDup (k, KEY_CP_NAME);
 	keyAddBaseName (testKey, keyString (k));
 
 	const Key * test = keyGetMeta (key, keyName (testKey));
@@ -1949,7 +1937,7 @@ KeySet * ksMetaGetSingleOrArray (Key * key, const char * metaName)
 	if (test == NULL)
 	{
 		// add dummy key to mimic elektraMetaArrayToKS
-		return ksNew (2, keyNew ("/#", KEY_END), k, KS_END);
+		return ksNew (2, keyNew ("meta:/#", KEY_END), k, KS_END);
 	}
 
 	return elektraMetaArrayToKS (key, metaName);
@@ -1960,11 +1948,11 @@ KeySet * ksMetaGetSingleOrArray (Key * key, const char * metaName)
  *
  * @return a newly allocated string, must be freed with elektraFree()
  */
-char * generateUsageLine (const char * progname, const Key * command, const Key * commandArgs)
+char * generateUsageLine (const char * progname, Key * command, const Key * commandArgs)
 {
-	size_t commandStringSize = keyGetUnescapedNameSize (commandArgs);
+	size_t commandStringSize = keyGetUnescapedNameSize (commandArgs) - 2;
 	char * commandString = elektraMalloc (commandStringSize);
-	memcpy (commandString, keyUnescapedName (commandArgs), commandStringSize);
+	memcpy (commandString, ((const char *) keyUnescapedName (commandArgs)) + 2, commandStringSize);
 
 	for (char * p = commandString; p < commandString + commandStringSize; ++p)
 	{
@@ -1983,7 +1971,7 @@ char * generateUsageLine (const char * progname, const Key * command, const Key
 	else
 	{
 		size_t argsTotalSize = 0;
-		for (cursor_t i = 1; i < ksGetSize (args); ++i) // start at one to skip size
+		for (elektraCursor i = 1; i < ksGetSize (args); ++i) // start at one to skip size
 		{
 			argsTotalSize += strlen (keyString (ksAtCursor (args, i))) + 3; // + 3 for space and []
 		}
@@ -1991,7 +1979,7 @@ char * generateUsageLine (const char * progname, const Key * command, const Key
 		indexedArgs = elektraMalloc (argsTotalSize + 1);
 		char * pos = indexedArgs;
 		size_t size = argsTotalSize + 1;
-		for (cursor_t i = 1; i < ksGetSize (args); i++) // start at one to skip size
+		for (elektraCursor i = 1; i < ksGetSize (args); i++) // start at one to skip size
 		{
 			*pos++ = '<';
 			pos = memccpy (pos, keyString (ksAtCursor (args, i)), '\0', size);
@@ -2011,25 +1999,26 @@ char * generateUsageLine (const char * progname, const Key * command, const Key
 		{
 			if (remainingArgs != NULL)
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...] [COMMAND [...]|%s [<%s>...]]\n", progname, commandString,
-						       indexedArgs, remainingArgs);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...] [COMMAND [...]|%s [<%s>...]]\n", progname,
+						       commandStringSize > 1 ? " " : "", commandString, indexedArgs, remainingArgs);
 			}
 			else
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...] [COMMAND [...]|%s]\n", progname, commandString,
-						       indexedArgs);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...] [COMMAND [...]|%s]\n", progname,
+						       commandStringSize > 1 ? " " : "", commandString, indexedArgs);
 			}
 		}
 		else
 		{
 			if (remainingArgs != NULL)
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...] [COMMAND [...]|[<%s>...]]\n", progname, commandString,
-						       remainingArgs);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...] [COMMAND [...]|[<%s>...]]\n", progname,
+						       commandStringSize > 1 ? " " : "", commandString, remainingArgs);
 			}
 			else
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...] [COMMAND [...]]\n", progname, commandString);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...] [COMMAND [...]]\n", progname,
+						       commandStringSize > 1 ? " " : "", commandString);
 			}
 		}
 	}
@@ -2039,23 +2028,26 @@ char * generateUsageLine (const char * progname, const Key * command, const Key
 		{
 			if (remainingArgs != NULL)
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...] %s [<%s>...]\n", progname, commandString, indexedArgs,
-						       remainingArgs);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...] %s [<%s>...]\n", progname,
+						       commandStringSize > 1 ? " " : "", commandString, indexedArgs, remainingArgs);
 			}
 			else
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...] %s\n", progname, commandString, indexedArgs);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...] %s\n", progname, commandStringSize > 1 ? " " : "",
+						       commandString, indexedArgs);
 			}
 		}
 		else
 		{
 			if (remainingArgs != NULL)
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...] [<%s>...]\n", progname, commandString, remainingArgs);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...] [<%s>...]\n", progname, commandStringSize > 1 ? " " : "",
+						       commandString, remainingArgs);
 			}
 			else
 			{
-				usage = elektraFormat ("Usage: %s%s [OPTION...]\n", progname, commandString);
+				usage = elektraFormat ("Usage: %s%s%s [OPTION...]\n", progname, commandStringSize > 1 ? " " : "",
+						       commandString);
 			}
 		}
 	}
@@ -2074,7 +2066,7 @@ char * generateUsageLine (const char * progname, const Key * command, const Key
  */
 char * generateOptionsList (KeySet * keysWithOpts, Key * commandKey)
 {
-	cursor_t cursor = ksGetCursor (keysWithOpts);
+	elektraCursor cursor = ksGetCursor (keysWithOpts);
 
 	char * optionsList = elektraStrDup ("");
 
@@ -2082,7 +2074,7 @@ char * generateOptionsList (KeySet * keysWithOpts, Key * commandKey)
 	ksRewind (keysWithOpts);
 	while ((cur = ksNext (keysWithOpts)) != NULL)
 	{
-		Key * checkKey = keyDup (cur);
+		Key * checkKey = keyDup (cur, KEY_CP_NAME);
 		if (strcmp (keyBaseName (cur), "#") == 0)
 		{
 			keySetBaseName (checkKey, NULL); // remove #
@@ -2124,7 +2116,7 @@ char * generateOptionsList (KeySet * keysWithOpts, Key * commandKey)
  */
 char * generateCommandsList (KeySet * keysWithOpts, Key * commandKey)
 {
-	cursor_t cursor = ksGetCursor (keysWithOpts);
+	elektraCursor cursor = ksGetCursor (keysWithOpts);
 
 	char * commandsList = elektraStrDup ("");
 
@@ -2132,7 +2124,7 @@ char * generateCommandsList (KeySet * keysWithOpts, Key * commandKey)
 	ksRewind (keysWithOpts);
 	while ((cur = ksNext (keysWithOpts)) != NULL)
 	{
-		Key * checkKey = keyDup (cur);
+		Key * checkKey = keyDup (cur, KEY_CP_NAME);
 		if (strcmp (keyBaseName (cur), "#") == 0)
 		{
 			keySetBaseName (checkKey, NULL); // remove #
@@ -2175,7 +2167,7 @@ char * generateCommandsList (KeySet * keysWithOpts, Key * commandKey)
  */
 char * generateArgsList (KeySet * keysWithOpts, Key * commandKey)
 {
-	cursor_t cursor = ksGetCursor (keysWithOpts);
+	elektraCursor cursor = ksGetCursor (keysWithOpts);
 
 	char * argsList = elektraStrDup ("");
 
@@ -2183,7 +2175,7 @@ char * generateArgsList (KeySet * keysWithOpts, Key * commandKey)
 	ksRewind (keysWithOpts);
 	while ((cur = ksNext (keysWithOpts)) != NULL)
 	{
-		Key * checkKey = keyDup (cur);
+		Key * checkKey = keyDup (cur, KEY_CP_NAME);
 		if (strcmp (keyBaseName (cur), "#") == 0)
 		{
 			keySetBaseName (checkKey, NULL); // remove #
@@ -2226,7 +2218,7 @@ char * generateArgsList (KeySet * keysWithOpts, Key * commandKey)
  */
 char * generateEnvsList (KeySet * keysWithOpts)
 {
-	cursor_t cursor = ksGetCursor (keysWithOpts);
+	elektraCursor cursor = ksGetCursor (keysWithOpts);
 
 	char * envsList = elektraStrDup ("");
 
