@@ -211,7 +211,7 @@ END_TEST ()
 BEGIN_TEST (output)
 char desired[] = {
 	";Ni1\n"
-	"; Generated by Nickel Plugin using Elektra (see libelektra.org).\n\n"
+	"; Generated by the ni plugin using Elektra (see libelektra.org).\n\n"
 	"1 = 1's value\n\n"
 	"[1]\n\n"
 	" [[2]]\n"
@@ -261,7 +261,7 @@ END_TEST ()
 BEGIN_TEST (output_modified)
 char desired[] = {
 	";Ni1\n"
-	"; Generated by Nickel Plugin using Elektra (see libelektra.org).\n\n"
+	"; Generated by the ni plugin using Elektra (see libelektra.org).\n\n"
 	"3 = 3's value\n"
 };
 
