@@ -370,13 +370,13 @@ static void test_keySetName (void)
 
 	succeed_if (keySetName (key, "/") != -1, "could not set cascading name");
 	succeed_if_same_string (keyName (key), "/");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "/");
 	keyDel (dup);
 
 	keySetName (key, "/c");
 	succeed_if_same_string (keyName (key), "/c");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "/c");
 	keyDel (dup);
 
@@ -384,43 +384,43 @@ static void test_keySetName (void)
 	succeed_if_same_string (keyName (key), "/");
 	keySetName (key, "/cascading");
 	succeed_if_same_string (keyName (key), "/cascading");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "/cascading");
 	keyDel (dup);
 
 	keySetName (key, "/cascading/s/deep/below");
 	succeed_if_same_string (keyName (key), "/cascading/s/deep/below");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "/cascading/s/deep/below");
 	keyDel (dup);
 
 	keySetName (key, "user:/cascading/s/deep/below");
 	succeed_if_same_string (keyName (key), "user:/cascading/s/deep/below");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "user:/cascading/s/deep/below");
 	keyDel (dup);
 
 	keySetName (key, "system:/cascading/s/deep/below");
 	succeed_if_same_string (keyName (key), "system:/cascading/s/deep/below");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "system:/cascading/s/deep/below");
 	keyDel (dup);
 
 	keySetName (key, "meta:/order");
 	succeed_if_same_string (keyName (key), "meta:/order");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "meta:/order");
 	keyDel (dup);
 
 	keySetName (key, "meta:/check/type");
 	succeed_if_same_string (keyName (key), "meta:/check/type");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "meta:/check/type");
 	keyDel (dup);
 
 	keySetName (key, "meta:/a");
 	succeed_if_same_string (keyName (key), "meta:/a");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "meta:/a");
 	keyDel (dup);
 
@@ -432,21 +432,21 @@ static void test_keySetName (void)
 
 	keySetName (key, "/cascading");
 	succeed_if_same_string (keyName (key), "/cascading");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "/cascading");
 	keyDel (dup);
 
 	keySetName (key, "meta:/");
 	succeed_if_same_string (keyName (key), "meta:/");
 	succeed_if (key->key != 0, "null pointer?");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "meta:/");
 	keyDel (dup);
 
 	keySetName (key, "meta:/other");
 	succeed_if_same_string (keyName (key), "meta:/other");
 	succeed_if (key->key != 0, "null pointer?");
-	dup = keyDup (key);
+	dup = keyDup (key, KEY_CP_ALL);
 	succeed_if_same_string (keyName (dup), "meta:/other");
 	keyDel (dup);
 
@@ -454,31 +454,31 @@ static void test_keySetName (void)
 	{
 		keySetName (key, "spec:/test");
 		succeed_if_same_string (keyName (key), "spec:/test");
-		dup = keyDup (key);
+		dup = keyDup (key, KEY_CP_ALL);
 		succeed_if_same_string (keyName (dup), "spec:/test");
 		keyDel (dup);
 
 		keySetName (key, "proc:/test");
 		succeed_if_same_string (keyName (key), "proc:/test");
-		dup = keyDup (key);
+		dup = keyDup (key, KEY_CP_ALL);
 		succeed_if_same_string (keyName (dup), "proc:/test");
 		keyDel (dup);
 
 		keySetName (key, "dir:/test");
 		succeed_if_same_string (keyName (key), "dir:/test");
-		dup = keyDup (key);
+		dup = keyDup (key, KEY_CP_ALL);
 		succeed_if_same_string (keyName (dup), "dir:/test");
 		keyDel (dup);
 
 		keySetName (key, "user:/test");
 		succeed_if_same_string (keyName (key), "user:/test");
-		dup = keyDup (key);
+		dup = keyDup (key, KEY_CP_ALL);
 		succeed_if_same_string (keyName (dup), "user:/test");
 		keyDel (dup);
 
 		keySetName (key, "system:/test");
 		succeed_if_same_string (keyName (key), "system:/test");
-		dup = keyDup (key);
+		dup = keyDup (key, KEY_CP_ALL);
 		succeed_if_same_string (keyName (dup), "system:/test");
 		keyDel (dup);
 	}
@@ -710,12 +710,12 @@ static void test_keyNeedSync (void)
 	succeed_if (keyNeedSync (k), "new meta, should definitely need sync");
 
 	clear_bit (k->flags, KEY_FLAG_SYNC);
-	Key * d = keyDup (k);
+	Key * d = keyDup (k, KEY_CP_ALL);
 	succeed_if (keyNeedSync (d), "dup key, should definitely need sync");
 
 	clear_bit (k->flags, KEY_FLAG_SYNC);
 	clear_bit (d->flags, KEY_FLAG_SYNC);
-	succeed_if (keyCopy (d, k) != -1, "copy not successful");
+	succeed_if (keyCopy (d, k, 0) != NULL, "copy not successful");
 	succeed_if (keyNeedSync (d), "copy key, should definitely need sync");
 	succeed_if (!keyNeedSync (k), "sources sync flag should not be affected");
 	keyDel (d);
@@ -752,16 +752,17 @@ static void test_keyNeedSync (void)
 
 static void test_keyCopy (void)
 {
+	// FIXME: add more tests
 	printf ("test copy key\n");
 	Key * k = keyNew ("/", KEY_END);
 	Key * c = keyNew ("user:/name", KEY_END);
 
-	succeed_if (keyCopy (c, k) != -1, "could not copy");
+	succeed_if (keyCopy (c, k, KEY_CP_NAME) != NULL, "could not copy");
 	succeed_if_same_string (keyName (k), "/");
 	succeed_if_same_string (keyName (c), "/");
 
 	succeed_if (keySetName (k, "/abc") != -1, "could not set cascading name");
-	succeed_if (keyCopy (c, k) != -1, "could not copy");
+	succeed_if (keyCopy (c, k, KEY_CP_NAME) != NULL, "could not copy");
 	succeed_if_same_string (keyName (k), "/abc");
 	succeed_if_same_string (keyName (c), "/abc");
 
@@ -870,6 +871,298 @@ static void test_warnings (void)
 	keyDel (key);
 }
 
+static void test_keyReplacePrefix (void)
+{
+	printf ("Test keyReplacePrefix\n");
+
+	Key * key = keyNew ("user:/", KEY_END);
+	Key * oldPrefix = keyNew ("user:/", KEY_END);
+	Key * newPrefix = keyNew ("user:/", KEY_END);
+
+	succeed_if (keyReplacePrefix (NULL, oldPrefix, newPrefix) == -1, "should not accept NULL argument");
+	succeed_if (keyReplacePrefix (key, NULL, newPrefix) == -1, "should not accept NULL argument");
+	succeed_if (keyReplacePrefix (key, oldPrefix, NULL) == -1, "should not accept NULL argument");
+
+	set_bit (key->flags, KEY_FLAG_RO_NAME);
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == -1, "should not accept read-only key");
+	clear_bit (key->flags, KEY_FLAG_RO_NAME);
+
+	keySetName (oldPrefix, "user:/foo");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 0, "shouldn't touch keys not below oldPrefix");
+	succeed_if (keyGetNameSize (key) == 7, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 10, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 7, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 3, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "user:/");
+	succeed_if_same_string (keyName (oldPrefix), "user:/foo");
+	succeed_if_same_string (keyName (newPrefix), "user:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_USER, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_USER, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0", 2) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "user:/bar");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 0, "shouldn't touch keys not below oldPrefix");
+	succeed_if (keyGetNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 10, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 7, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 6, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "user:/bar");
+	succeed_if_same_string (keyName (oldPrefix), "user:/foo");
+	succeed_if_same_string (keyName (newPrefix), "user:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_USER, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_USER, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0bar", 5) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/foo/bar");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 0, "shouldn't touch keys with different namespace than oldPrefix");
+	succeed_if (keyGetNameSize (key) == 16, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 10, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 7, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "system:/foo/bar");
+	succeed_if_same_string (keyName (oldPrefix), "user:/foo");
+	succeed_if_same_string (keyName (newPrefix), "user:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_SYSTEM, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_USER, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0foo\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "/foo/bar");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 0, "shouldn't touch keys with different namespace than oldPrefix");
+	succeed_if (keyGetNameSize (key) == 9, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 10, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 7, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "/foo/bar");
+	succeed_if_same_string (keyName (oldPrefix), "user:/foo");
+	succeed_if_same_string (keyName (newPrefix), "user:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_CASCADING, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_USER, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0foo\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/foo/bar");
+	keySetName (oldPrefix, "/foo");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 0, "shouldn't touch keys with different namespace than oldPrefix");
+	succeed_if (keyGetNameSize (key) == 16, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 5, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 7, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "system:/foo/bar");
+	succeed_if_same_string (keyName (oldPrefix), "/foo");
+	succeed_if_same_string (keyName (newPrefix), "user:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_SYSTEM, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_CASCADING, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0foo\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/foo/bar");
+	keySetName (oldPrefix, "system:/foo");
+	keySetName (newPrefix, "system:/baz");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 16, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 12, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 12, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 6, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "system:/baz/bar");
+	succeed_if_same_string (keyName (oldPrefix), "system:/foo");
+	succeed_if_same_string (keyName (newPrefix), "system:/baz");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_SYSTEM, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_SYSTEM, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_SYSTEM, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0baz\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0baz", 5) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/foo/bar");
+	keySetName (oldPrefix, "system:/foo");
+	keySetName (newPrefix, "user:/baz");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 14, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 12, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 10, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 6, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "user:/baz/bar");
+	succeed_if_same_string (keyName (oldPrefix), "system:/foo");
+	succeed_if_same_string (keyName (newPrefix), "user:/baz");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_USER, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_SYSTEM, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0baz\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0baz", 5) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/foo/bar");
+	keySetName (oldPrefix, "system:/foo/bar");
+	keySetName (newPrefix, "system:/baz");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 12, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 16, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 12, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 6, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 10, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 6, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "system:/baz");
+	succeed_if_same_string (keyName (oldPrefix), "system:/foo/bar");
+	succeed_if_same_string (keyName (newPrefix), "system:/baz");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_SYSTEM, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_SYSTEM, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_SYSTEM, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0baz", 5) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo\0bar", 9) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0baz", 5) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/foo/bar");
+	keySetName (oldPrefix, "system:/");
+	keySetName (newPrefix, "user:/baz");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 18, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 9, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 10, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 14, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 3, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 6, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "user:/baz/foo/bar");
+	succeed_if_same_string (keyName (oldPrefix), "system:/");
+	succeed_if_same_string (keyName (newPrefix), "user:/baz");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_USER, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_SYSTEM, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0baz\0foo\0bar", 13) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0", 2) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0baz", 5) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/foo/bar");
+	keySetName (oldPrefix, "system:/");
+	keySetName (newPrefix, "user:/");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 14, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 9, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 7, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 3, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "user:/foo/bar");
+	succeed_if_same_string (keyName (oldPrefix), "system:/");
+	succeed_if_same_string (keyName (newPrefix), "user:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_USER, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_SYSTEM, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0foo\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0", 2) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "system:/");
+	keySetName (oldPrefix, "system:/");
+	keySetName (newPrefix, "user:/");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 7, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 9, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 7, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 3, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 3, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "user:/");
+	succeed_if_same_string (keyName (oldPrefix), "system:/");
+	succeed_if_same_string (keyName (newPrefix), "user:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_USER, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_SYSTEM, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_USER, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0", 2) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0", 2) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "/foo/bar");
+	keySetName (oldPrefix, "/foo");
+	keySetName (newPrefix, "/baz");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 9, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 5, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 5, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 6, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 6, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "/baz/bar");
+	succeed_if_same_string (keyName (oldPrefix), "/foo");
+	succeed_if_same_string (keyName (newPrefix), "/baz");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_CASCADING, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_CASCADING, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_CASCADING, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0baz\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0foo", 5) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0baz", 5) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "/foo/bar");
+	keySetName (oldPrefix, "/");
+	keySetName (newPrefix, "system:/baz");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 20, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 2, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 12, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 14, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 3, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 6, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "system:/baz/foo/bar");
+	succeed_if_same_string (keyName (oldPrefix), "/");
+	succeed_if_same_string (keyName (newPrefix), "system:/baz");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_SYSTEM, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_CASCADING, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_SYSTEM, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0baz\0foo\0bar", 13) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0", 2) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0baz", 5) == 0, "newPrefix has wrong unescaped name");
+
+	keySetName (key, "/foo/bar");
+	keySetName (oldPrefix, "/");
+	keySetName (newPrefix, "system:/");
+	succeed_if (keyReplacePrefix (key, oldPrefix, newPrefix) == 1, "didn't correctly replace");
+	succeed_if (keyGetNameSize (key) == 16, "key size wrong");
+	succeed_if (keyGetNameSize (oldPrefix) == 2, "oldPrefix size wrong");
+	succeed_if (keyGetNameSize (newPrefix) == 9, "newPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (key) == 10, "key size wrong");
+	succeed_if (keyGetUnescapedNameSize (oldPrefix) == 3, "oldPrefix size wrong");
+	succeed_if (keyGetUnescapedNameSize (newPrefix) == 3, "newPrefix size wrong");
+	succeed_if_same_string (keyName (key), "system:/foo/bar");
+	succeed_if_same_string (keyName (oldPrefix), "/");
+	succeed_if_same_string (keyName (newPrefix), "system:/");
+	succeed_if (*(char *) keyUnescapedName (key) == KEY_NS_SYSTEM, "key has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (oldPrefix) == KEY_NS_CASCADING, "oldPrefix has wrong namespace");
+	succeed_if (*(char *) keyUnescapedName (newPrefix) == KEY_NS_SYSTEM, "newPrefix has wrong namespace");
+	succeed_if (memcmp ((char *) keyUnescapedName (key) + 1, "\0foo\0bar", 9) == 0, "key has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (oldPrefix) + 1, "\0", 2) == 0, "oldPrefix has wrong unescaped name");
+	succeed_if (memcmp ((char *) keyUnescapedName (newPrefix) + 1, "\0", 2) == 0, "newPrefix has wrong unescaped name");
+
+	keyDel (key);
+	keyDel (oldPrefix);
+	keyDel (newPrefix);
+}
+
 int main (int argc, char ** argv)
 {
 	printf ("KEY      TESTS\n");
@@ -893,6 +1186,7 @@ int main (int argc, char ** argv)
 	test_keyFixedNew ();
 	test_keyFlags ();
 	test_warnings ();
+	test_keyReplacePrefix ();
 
 	print_result ("test_key");
 	return nbError;
